// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: This program exercises the handling and throwing
//				  of java API and user-generated exceptions
//				  (i.e., exception objects).
//				  
// Version		: 1.0

import java.util.Scanner;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.InputMismatchException;

public class ExceptionsDemo {
	
	// Method "main" drives topic selection
	
	public static void main(String[] args) {
		ExceptionsDemo demoObject = new ExceptionsDemo();
		
		// Display menu and wait for a menu item selection
		
		Scanner menuSelect = new Scanner(System.in);
		
		// Initialize menuItem selection.
		int menuItem = 0;
		
		while (menuItem != 9) {
			// Display menu
			System.out.println("\n***Exceptions Demo Menu ***");
			System.out.println("1 - Catch exceptions");
			System.out.println("2 - Catch java API exceptions");
			System.out.println("3 - Throw exceptions");
			System.out.println("9 - Exit");
			System.out.println();
			System.out.println("Enter demo number:");
			
			// Input a menu selection.
			menuItem = menuSelect.nextInt();
			
			// Run demo corresponding to selected menu item.
			switch (menuItem) {
				
				case 1 : {
					System.out.println("\n*** Running catch exceptions demo ***\n");
					
					// Execute catch exceptions demo.
					demoObject.catchExceptionDemo();
					break;
				}
				
				case 2 : {
					System.out.println("\n*** Running catch API exceptions demo ***\n");
					
					// Execute catch API exceptions demo.
					demoObject.catchAPIExceptionsDemo();
					break;
				}
				
				case 3 : {
					System.out.println("\n*** Running throw exceptions demo ***\n");
					
					// Execute throw exceptions demo.
					demoObject.throwExceptionDemo();
					break;
				}
		
			 // case n :     More upcoming demos ...
				
				default : {
					if (menuItem != 9)
						System.out.println("Invalid selection entered");
					break;
				}
			}
			
			if (menuItem != 9) 	
				System.out.println("\n*** Demo completed ***\n");
		}
		
		System.out.println("\n*** Program terminated ***\n");	
	}
	
	// Catch exceptions demo.
	
	public void catchExceptionDemo() {

		// Declare a data stream object to read data from a text file.
		Scanner fileInputData = null;
		
		// Used to keep track of "file found" action.
		boolean found = false;
			
		// Open the file. Check for file I/O exceptions.
		try {
			// Attempt to open the file for reading.
			fileInputData = new Scanner(new FileInputStream("test.txt"));
			
			// Flag the file as found.
			found = true;
			
			// If successful, print the first line of the file.
			System.out.println("<<< " + fileInputData.nextLine() + " >>>");
		}
		catch(FileNotFoundException e) {	
			System.out.println("Error opening file 'test.txt'");
		}
		// catch (IOException e1) {
			// System.out.println("An I/O error has occurred");
		// }
		finally {
			
			if (!found) {
				System.out.println("File 'test.txt' was not found. Program aborted!");
				System.exit(0);
			}
			else
				System.out.println("\nSuccessfully read first line of test.txt!");
		}
	}
	
	public void catchAPIExceptionsDemo() {
		// The 'Exception' class is the base class for exceptions.
		// Other examples are: IOException, NoSuchMethodException,
		// InputMismatchException, FileNotFoundException, etc.
		
		// Declare a keyboard reader object, a loop control flag, and
		// a desired integer to be enter from the keyboard.
		Scanner keyboard = new Scanner(System.in);
		boolean notDone = true;
		int controlValue;
		
		// Loop until an "integer value" is entered. Handle the invalid
		// input instances by throwing and catching exceptions.
		
		while (notDone) {
			try {
				System.out.print("Enter an integer value: ");
				
				// Method invocation "keyboard.nextInt()" might throw an Exception.
				controlValue = keyboard.nextInt();
				
				// The entered value is fine. Terminate the loop.
				System.out.println("Entered value '" + controlValue + "' is O.K.");
				notDone = false; 
			}
			catch (Exception e) {
				keyboard.nextLine();
				System.out.println("Entered value should be an integer number! Try again!");
			};
		}
		
		// Loop until a valid control code 'A1' is entered. Handle the
		// invalid code situations by throwing an exception in the "try"
		// block and catching it in the "cath" block.
		
		notDone = true;
		String controlString;
		while (notDone) {
			try {
				System.out.print("\nEnter a control key (A1): ");
				controlString = keyboard.next();
				
				// Compare the String object read from the keyboard in
				// "controlString" with the valid String "A1". If they
				// are different throw an InputMismatchException.
				if (!(controlString.equals("A1")))
					throw new InputMismatchException("Invalid control key");
				
				// The entered string is fine. Terminate the loop.
				System.out.println("Entered key '" + controlString + "' is O.K.");
				notDone = false; 
			}
			catch (InputMismatchException e) {
				keyboard.nextLine();
				System.out.println(e.getMessage().toUpperCase());
				System.out.println("Try again!");
				
			};
		}
		
		// Loop until a valid control string 'A2' is entered. No throw
		// statement in "try" block. However invocation of method 
		// "getControlString" might throw an InputMismatchException.
		// Therefore, this eception  has to be handled in a "catch" block.
		
		notDone = true;
		while (notDone) {
			try {
				// Get control dtring using a user-defined method.
				// Method "getControlString" can issue an InputMismatchException.
				controlString = getControlString(keyboard);
	
				// The entered string is fine. Terminate the loop.
				System.out.println("Entered key '" + controlString + "' is O.K.");
				notDone = false; 
			}
			catch (InputMismatchException e) {
				keyboard.nextLine();
				System.out.println(e.getMessage().toUpperCase());
				System.out.println("Try again!");
			}
		}
	}
	
	// Throw user-generated exceptions demo.
	
	public void throwExceptionDemo() {	
		double temperature;
		try {
			// Invoke method "readCoreTemp" which might throw a user-generated
			// OutOfRangeException.
			temperature = readCoreTemp();
		}
		catch (OutOfRangeException e2) {
			System.out.println(e2.getMessage());
		}
	}
	
	private String getControlString(Scanner keyboard) throws InputMismatchException {
		System.out.print("\nEnter a control string (A2): ");
		String controlString = keyboard.next();
				
		if (!(controlString.equals("A2")))
			throw new InputMismatchException("Invalid control string");
		
		return controlString;
	}	
	
	private double readCoreTemp() throws OutOfRangeException {
		// Assume that this section of code reads the core's temperature
		// in a nuclear reactor, and loads it into variable "temperature".
		// This is simulated just by loading a value in "temperature".
		
		// temperature loaded with 500, in order to throw an OutOfRangeException
		double temperature = 500;
		
		// Checks that the temperature doesn't exceed 400 Celsius degrees.
		if (temperature > 400)
			throw new OutOfRangeException("Core's temperature = " + temperature + 
										  " is unsafe!!! Maximum should be 400 C");
		return temperature;
	}
	
}